package kubernetes

container_resources[r] {
    r := input
    input.kind == "Deployment"
}

container_resources[r] {
    r := input
    input.kind == "Job"
}

container_resources[r] {
    r := input
    input.kind == "Pod"
}

# Containers for Jobs/Pods/Deployments should have containers[_].resources.requests configured for CPU & Memory
# Pod CPU Version
resource_missing_container_resources_requests[reason] {
    r := container_resources[_]
    containers := r.spec.containers[_]
    not containers.resources.requests.cpu
    reason := sprintf("%s %s: Container %s is missing container CPU resource request configuration. Set resources.requests.cpu", [r.kind, r.metadata.name, containers.name])
}

# Pod Memory Version
resource_missing_container_resources_requests[reason] {
    r := container_resources[_]
    containers := r.spec.containers[_]
    not containers.resources.requests.memory
    reason := sprintf("%s %s: Container %s is missing container Memory resource request configuration. Set resources.requests.memory", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs CPU Version
resource_missing_container_resources_requests[reason] {
    r := container_resources[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.requests.cpu
    reason := sprintf("%s %s: Container %s is missing container CPU resource request configuration. Set resources.requests.cpu", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Memory Version
resource_missing_container_resources_requests[reason] {
    r := container_resources[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.requests.memory
    reason := sprintf("%s %s: Container %s is missing container Memory resource request configuration. Set resources.requests.memory", [r.kind, r.metadata.name, containers.name])
}

# Actual Test Cases
deny_resource_missing_container_resources_requests[msg]{
  not array_contains(warning_mode, "resource_missing_container_resources_requests")
  resource_missing_container_resources_requests[_] != []
  msg := resource_missing_container_resources_requests[_]
}

warn_resource_missing_container_resources_requests[msg]{
  array_contains(warning_mode, "resource_missing_container_resources_requests")
  resource_missing_container_resources_requests[_] != []
  msg := resource_missing_container_resources_requests[_]
}

# Containers for Jobs/Pods/Deployments should have containers[_].resources.limits configured for CPU & Memory
# Pod CPU Version
resource_missing_container_resources_limits[reason] {
    r := container_resources[_]
    containers := r.spec.containers[_]
    not containers.resources.limits.cpu
    reason := sprintf("%s %s: Container %s is missing container CPU resource request configuration. Set resources.limits.cpu", [r.kind, r.metadata.name, containers.name])
}

# Pod Memory Version
resource_missing_container_resources_limits[reason] {
    r := container_resources[_]
    containers := r.spec.containers[_]
    not containers.resources.limits.memory
    reason := sprintf("%s %s: Container %s is missing container Memory resource request configuration. Set resources.limits.memory", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs CPU Version
resource_missing_container_resources_limits[reason] {
    r := container_resources[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.limits.cpu
    reason := sprintf("%s %s: Container %s is missing container CPU resource request configuration. Set resources.limits.cpu", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Memory Version
resource_missing_container_resources_limits[reason] {
    r := container_resources[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.limits.memory
    reason := sprintf("%s %s: Container %s is missing container Memory resource request configuration. Set resources.limits.memory", [r.kind, r.metadata.name, containers.name])
}

# Actual Test Cases
deny_resource_missing_container_resources_limits[msg]{
  not array_contains(warning_mode, "resource_missing_container_resources_limits")
  resource_missing_container_resources_limits[_] != []
  msg := resource_missing_container_resources_limits[_]
}

warn_resource_missing_container_resources_limits[msg]{
  array_contains(warning_mode, "resource_missing_container_resources_limits")
  resource_missing_container_resources_limits[_] != []
  msg := resource_missing_container_resources_limits[_]
}