package kubernetes

deployments[r] {
    r := input
    input.kind == "Deployment"
}

# Deployments should have readiness probes configured
deployment_missing_readiness_probe[reason] {
    r := deployments[_]
    containers := r.spec.template.spec.containers[_]
    not containers.readinessProbe
    reason := sprintf("%s %s: Container %s is missing a Readiness Probe configuration. Readiness probes will tell you when your app is ready to serve traffic.", [r.kind, r.metadata.name, containers.name])
}

deny_deployment_missing_readiness_probe[msg]{
  not array_contains(warning_mode, "deployment_missing_readiness_probe")
  deployment_missing_readiness_probe[_] != []
  msg := deployment_missing_readiness_probe[_]
}

warn_deployment_missing_readiness_probe[msg]{
  array_contains(warning_mode, "deployment_missing_readiness_probe")
  deployment_missing_readiness_probe[_] != []
  msg := deployment_missing_readiness_probe[_]
}

# Deployments should have livenss probes configured
deployment_missing_liveness_probe[reason] {
    r := deployments[_]
    containers := r.spec.template.spec.containers[_]
    not containers.livenessProbe
    reason := sprintf("%s %s: Container %s is missing a Liveness Probe configuration. Liveness probes will let Kubernetes know whether your app is healthy.", [r.kind, r.metadata.name, containers.name])
}

deny_deployment_missing_liveness_probe[msg]{
  not array_contains(warning_mode, "deployment_missing_liveness_probe")
  deployment_missing_liveness_probe[_] != []
  msg := deployment_missing_liveness_probe[_]
}

warn_deployment_missing_liveness_probe[msg]{
  array_contains(warning_mode, "deployment_missing_liveness_probe")
  deployment_missing_liveness_probe[_] != []
  msg := deployment_missing_liveness_probe[_]
}

# Deployments should have the imagePullPolicy set to Always
deployment_image_pull_policy[reason] {
    r := deployments[_]
    containers := r.spec.template.spec.containers[_]
    not containers.imagePullPolicy == "Always"
    reason := sprintf("%s %s: Container %s imagePullPolicy should be set to 'Always'.", [r.kind, r.metadata.name, containers.name])
}

deny_deployment_image_pull_policy[msg]{
  not array_contains(warning_mode, "deployment_image_pull_policy")
  deployment_image_pull_policy[_] != []
  msg := deployment_image_pull_policy[_]
}

warn_deployment_image_pull_policy[msg]{
  array_contains(warning_mode, "deployment_image_pull_policy")
  deployment_image_pull_policy[_] != []
  msg := deployment_image_pull_policy[_]
}

# Deployments should have an unprivileged security context set
deployment_security_context_privileged[reason] {
    r := deployments[_]
    containers := r.spec.template.spec.containers[_]
    containers.securityContext["privileged"] == true
    reason := sprintf("%s %s: Container %s securityContext.privileged should not be set to 'true'.", [r.kind, r.metadata.name, containers.name])
}

deployment_security_context_privileged[reason] {
    r := deployments[_]
    r.spec.template.spec.securityContext["privileged"] == true
    reason := sprintf("%s %s: securityContext.privileged should not be set to 'true'.", [r.kind, r.metadata.name])
}

deny_deployment_security_context_privileged[msg]{
  not array_contains(warning_mode, "deployment_security_context_privileged")
  deployment_security_context_privileged[_] != []
  msg := deployment_security_context_privileged[_]
}

warn_deployment_security_context_privileged[msg]{
  array_contains(warning_mode, "deployment_security_context_privileged")
  deployment_security_context_privileged[_] != []
  msg := deployment_security_context_privileged[_]
}