package kubernetes

jobs[r] {
    r := input
    input.kind == "Job"
}

# Jobs should have the imagePullPolicy set to Always
job_image_pull_policy[reason] {
    r := jobs[_]
    containers := r.spec.template.spec.containers[_]
    not containers.imagePullPolicy == "Always"
    reason := sprintf("%s %s: Container %s imagePullPolicy should be set to 'Always'.", [r.kind, r.metadata.name, containers.name])
}

deny_job_image_pull_policy[msg]{
  not array_contains(warning_mode, "job_image_pull_policy")
  job_image_pull_policy[_] != []
  msg := job_image_pull_policy[_]
}

warn_job_image_pull_policy[msg]{
  array_contains(warning_mode, "job_image_pull_policy")
  job_image_pull_policy[_] != []
  msg := job_image_pull_policy[_]
}

# Job containers should have an unprivileged security context set
job_security_context_privileged[reason] {
    r := jobs[_]
    containers := r.spec.template.spec.containers[_]
    containers.securityContext["privileged"] == true
    reason := sprintf("%s %s: Container %s securityContext.privileged should not be set to 'true'.", [r.kind, r.metadata.name, containers.name])
}

job_security_context_privileged[reason] {
    r := jobs[_]
    r.spec.template.spec.securityContext["privileged"] == true
    reason := sprintf("%s %s: securityContext.privileged should not be set to 'true'.", [r.kind, r.metadata.name])
}

deny_job_security_context_privileged[msg]{
  not array_contains(warning_mode, "job_security_context_privileged")
  job_security_context_privileged[_] != []
  msg := job_security_context_privileged[_]
}

warn_job_security_context_privileged[msg]{
  array_contains(warning_mode, "job_security_context_privileged")
  job_security_context_privileged[_] != []
  msg := job_security_context_privileged[_]
}