package kubernetes

pods[r] {
    r := input
    input.kind == "Pod"
}

# Pods should have readiness probes configured
pod_missing_readiness_probe[reason] {
    r := pods[_]
    containers := r.spec.containers[_]
    not containers.readinessProbe
    reason := sprintf("%s %s: Container %s is missing a Readiness Probe configuration. Readiness probes will tell you when your app is ready to serve traffic.", [r.kind, r.metadata.name, containers.name])
}

deny_pod_missing_readiness_probe[msg]{
  not array_contains(warning_mode, "pod_missing_readiness_probe")
  pod_missing_readiness_probe[_] != []
  msg := pod_missing_readiness_probe[_]
}

warn_pod_missing_readiness_probe[msg]{
  array_contains(warning_mode, "pod_missing_readiness_probe")
  pod_missing_readiness_probe[_] != []
  msg := pod_missing_readiness_probe[_]
}

# Pods should have livenss probes configured
pod_missing_liveness_probe[reason] {
    r := pods[_]
    containers := r.spec.containers[_]
    not containers.livenessProbe
    reason := sprintf("%s %s: Container %s is missing a Liveness Probe configuration. Liveness probes will let Kubernetes know whether your app is healthy.", [r.kind, r.metadata.name, containers.name])
}

deny_pod_missing_liveness_probe[msg]{
  not array_contains(warning_mode, "pod_missing_liveness_probe")
  pod_missing_liveness_probe[_] != []
  msg := pod_missing_liveness_probe[_]
}

warn_pod_missing_liveness_probe[msg]{
  array_contains(warning_mode, "pod_missing_liveness_probe")
  pod_missing_liveness_probe[_] != []
  msg := pod_missing_liveness_probe[_]
}

# Pods should have the imagePullPolicy set to Always
pod_image_pull_policy[reason] {
    r := pods[_]
    containers := r.spec.containers[_]
    not containers.imagePullPolicy == "Always"
    reason := sprintf("%s %s: Container %s imagePullPolicy should be set to 'Always'.", [r.kind, r.metadata.name, containers.name])
}

deny_pod_image_pull_policy[msg]{
  not array_contains(warning_mode, "pod_image_pull_policy")
  pod_image_pull_policy[_] != []
  msg := pod_image_pull_policy[_]
}

warn_pod_image_pull_policy[msg]{
  array_contains(warning_mode, "pod_image_pull_policy")
  pod_image_pull_policy[_] != []
  msg := pod_image_pull_policy[_]
}

# Pods should have an unprivileged security context set
pod_security_context_privileged[reason] {
    r := pods[_]
    containers := r.spec.containers[_]
    containers.securityContext["privileged"] == true
    reason := sprintf("%s %s: Container %s securityContext.privileged should not be set to 'true'.", [r.kind, r.metadata.name, containers.name])
}

deny_pod_security_context_privileged[msg]{
  not array_contains(warning_mode, "pod_security_context_privileged")
  pod_security_context_privileged[_] != []
  msg := pod_security_context_privileged[_]
}

warn_pod_security_context_privileged[msg]{
  array_contains(warning_mode, "pod_security_context_privileged")
  pod_security_context_privileged[_] != []
  msg := pod_security_context_privileged[_]
}